$:.unshift File::dirname(__FILE__)
require 'mail_agent/models'
require 'mail_agent/ygg'
require 'mail_agent/version'

module MailAgent
  class BulkSync
    attr_accessor :domains, :boxes, :relext
    def initialize
      load_data
    end

    def load_data
      self.domains ||= Ygg::SimpleService::Email::Domain.view(:provisioning).all
      self.boxes ||= Ygg::SimpleService::Email::Box.view(:provisioning).all
      self.relext ||= Ygg::SimpleService::Email::RelayExtension.view(:provisioning).all
    end

    def sync
      load_data
      sync_domains
      sync_boxes
      sync_policyd
    rescue 
      ex = $!
      if defined?(Raven)
        Raven.capture_exception(ex)
      end
      raise ex
    end
      

    protected

    def sync_domains
      Models::Email::Domain.transaction do
        ygg_pks = []
        domains.each do | domain | 
          ygg_pks << domain.uuid if domain.create_or_update_db_domain!
        end

        db_pks = Models::Email::Domain.all.map(&:uuid)
        old = db_pks - ygg_pks

        if old.any?
          old = Models::Email::Domain.where(uuid: old)
          doms = old.map(&:domain).map{|d| "@#{d}"}
          Models::Amavis::User.where(email: doms).map(&:destroy)
          old.map(&:destroy!)
        end 
      end
    end

    def sync_boxes
      Models::Email::Box.transaction do
        ygg_pks = []
        boxes.each do | box |
          ygg_pks << box.uuid if box.create_or_update_db_box!
        end

        relext.each do | ext |
          ygg_pks << ext.uuid if ext.create_or_update_db_box!
        end

        db_pks = Models::Email::Box.all.map(&:uuid)
        old = db_pks - ygg_pks

        if old.any?
          old = Models::Email::Box.where(uuid: old)
          fqdas = old.map(&:fqda)
          Models::Amavis::User.where(email: fqdas).map(&:destroy)
          old.map(&:destroy!)
        end 
      end
    end

    def sync_policyd
      Models::Policyd::Policy.transaction do
        
        uuids = relext.inject({identities: [], addresses: []}) do | m, ext |
          if ext.create_or_update_db_extension!
            m[:identities] << "hel:#{ext.uuid}" if ext.is_identity?
            m[:addresses] << "hel:#{ext.uuid}" if ext.is_address?
          end
          m
        end
        
        uuids.each do | k, list |
          if list.any?
            case(k)
            when :identities then Models::Policyd::AuthenticatedIdentity.where.not(notes: list).destroy_all
            when :addresses then Models::Policyd::ClientAddress.where.not(notes: list).destroy_all
            end
          end
        end
      end    
    end
  end
end