
module MailAgent
  module Models
    module Cluebringer
      class PolicyGroup < MailAgent::Models::Cluebringer::Base
        self.primary_key = 'ID'
        self.table_name = 'policy_groups'

        POLICIES = {
          'default_for_identities' => { auth: 'bulk_auth_senders',     anon: 'bulk_ip_senders'     },
          'usr_128_h' =>              { auth: 'bulk_auth_senders_128', anon: 'bulk_ip_senders_128' },
          'usr_512_h' =>              { auth: 'bulk_auth_senders_512', anon: 'bulk_ip_senders_512' },
          'usr_1k_h' =>               { auth: 'bulk_auth_senders_1k',  anon: 'bulk_ip_senders_1k'  },
          'usr_4k_h' =>               { auth: 'bulk_auth_senders_4k',  anon: 'bulk_ip_senders_4k'  },
          'usr_12k_h' =>              { auth: 'bulk_auth_senders_12k', anon: 'bulk_ip_senders_12k' },
        }

        def self.build_default_policy_groups!
          POLICIES.each do | _, v |
            v.values.each do | policy |
              r = self.find_or_create_by(Name: policy)
              r.Disabled = false
              r.save!
            end
          end
        end

        has_many :members, 
          class_name: 'MailAgent::Models::Cluebringer::PolicyGroupMember',
          primary_key: 'ID',
          foreign_key: 'PolicyGroupID'

        def self.get(policy, anon = false)
          policies = POLICIES[policy.to_s]
          policy = (anon ? policies[:anon] : policies[:auth]) if policies
          raise ArgumentError.new "policy not found: \"#{policy}\", anonymous: #{anon.inspect}" unless policy
          grp = self.find_by(name: policy)
          raise ArgumentError.new "policy not found: \"#{policy}\", anonymous: #{anon.inspect}" unless grp
          grp
        end

        def self.build_default!
          POLICIES.each do | k, list |
            list.values.each do | pol |
              record = self.find_or_initialize_by Name: pol
              record.Disabled = false
              record.save!
            end
          end
        end

        def policy_name
          POLICIES.find{|k,v| v.values.include? self.Name}.try(&:first)
        end
      end
    end
  end
end
