
module MailAgent
  module Models
    module Cluebringer
      class PolicyGroupMember < MailAgent::Models::Cluebringer::Base
        self.primary_key = 'ID'
        self.table_name = 'policy_group_members'

        belongs_to :group, 
          class_name: 'MailAgent::Models::Cluebringer::PolicyGroup',
          primary_key: 'ID',
          foreign_key: 'PolicyGroupID'

        scope :managed, -> { where arel_table[:Comment].matches('hel:%')  }

        before_destroy :reset_quota_trackings
        
        def self.find_by_hel(object)
          self.managed.where_uuid(object.uuid).first
        end

        def self.where_uuid(uuid)
          uuid = [uuid].flatten
          uuid = uuid.map{|u| u ? "hel:#{u}" : u}
          self.where(Comment: uuid)
        end

        def self.create_or_update_with_hel!(ext)
          raise "#{ext.to_s} isn't a Ygg::SimpleService::Email::RelayExtension" unless ext.is_a?(Ygg::SimpleService::Email::RelayExtension)
          pg = PolicyGroup.get(ext.rate_limit, ext.is_address?)
          member = if ext.is_address?  
            ext.ip
          elsif ext.is_identity? 
            "$#{ext.login}"
          end

          pgm = self.find_or_initialize_by(Member: member) # maybe exists a manual entry
          
          unless pgm.persisted? && pgm.PolicyGroupID == pg.ID
            key = if ext.is_address?
              "SenderIP:#{ext.ip}/32" unless ext.ip =~ /\//
              "SenderIP:#{ext.ip}" if ext.ip =~ /\//
            else
              "SASLUsername:#{ext.login}"
            end
            QuotasTracking.where(TrackKey: key).delete_all if key
          end
          
          pgm.attributes = {
            PolicyGroupID: pg.ID,
            Member: member,
            Disabled: !!(ext.on_hold || ext.domain_on_hold),
            Comment: "hel:#{ext.uuid}"
          }

          pgm.save!
          pgm
        end

        def uuid
          self.Comment.gsub(/\Ahel:/, '') if self.Comment =~ /\Ahel:/
        end

        protected

        def reset_quota_trackings
          member = self.Member.gsub(/\A\$/, '')
          key = if member =~ /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/
            "SenderIP:#{member}/32" unless member =~ /\//
            "SenderIP:#{member}" if member =~ /\//
          else
            "SASLUsername:#{member}"
          end
          QuotasTracking.where(TrackKey: key).delete_all if key
        end
      end
    end
  end
end
