require 'securerandom'

module MailAgent
  module Models
    module Email
      class Box < Email::Base
        has_one :forward, primary_key: :uuid, foreign_key: :uuid, dependent: :destroy

        def self.find_by_hel(object)
          self.where(uuid: object.uuid).first
        end

        def self.create_or_update_with_hel!(object)
          case object
          when Ygg::SimpleService::Email::Box then create_or_update_with_hel_box!(object)
          when Ygg::SimpleService::Email::RelayExtension then create_or_update_with_hel_relay_extension!(object)
          else raise "Unhandled object #{object.class.name}"
          end
        end

        protected

        def self.create_or_update_with_hel_box!(object)
          raise "#{object.to_s} isn't a Ygg::SimpleService::Email::Box" unless object.is_a?(Ygg::SimpleService::Email::Box)

          record = self.find_or_initialize_by(uuid: object.uuid)

          names = self.column_names.map(&:to_sym)
          names -= [:id, :domain_on_hold, :password, :encrypted_password, :on_hold, :active]
          record.attributes = object.as_json(only: names).symbolize_keys
          record.on_hold = !object.active

          if object.encrypted_password.present?
            record.password = password_with_scheme(object.encrypted_password)
          elsif object.password.present?
            record.password = password_with_scheme(object.password)
          else
            record.password = password_with_scheme(SecureRandom.hex)
          end

          record.send_only = false
          record.save!

          if record.forward.present? && object.forwards.blank?
            record.forward.destroy
          elsif object.forwards.present?
            record.forward = Forward.create_or_update_with_hel!(object)
          end

          record
        end

        def self.create_or_update_with_hel_relay_extension!(object)
          raise "#{object.to_s} isn't a Ygg::SimpleService::Email::RelayExtension" unless object.is_a?(Ygg::SimpleService::Email::RelayExtension)
          record = self.where(uuid: object.uuid).first
          record.destroy if record && !object.is_credential?
          return false unless object.is_credential?

          record ||= new
          record.fqda = object.login
          record.user = object.login.split(/@/).first
          record.domain = object.login.split(/@/).last
          record.on_hold = !object.active
          record.send_only = !object.real_box
          record.uuid = object.uuid

          if object.encrypted_password.present?
            record.password = password_with_scheme(object.encrypted_password)
          elsif object.secret.present?
            record.password = password_with_scheme(object.secret)
          else
            record.password = password_with_scheme(SecureRandom.hex)
          end

          record.save!
          record
        end

        protected

        def self.password_with_scheme(pw)
          if pw =~ /\A\$(argon.*?)\$/
            "{#{$1.upcase}}#{pw}"
          else
            "{PLAIN}#{pw}"
          end
        end
      end
    end
  end
end
