
module MailAgent
  module Models
    module Email
      class Domain < Email::Base
        has_many :forwards, primary_key: :uuid, foreign_key: :uuid, dependent: :destroy

        def self.find_by_hel(object)
          self.where(uuid: object.uuid).first
        end

        def self.create_or_update_with_hel!(object)
          raise "#{object.to_s} isn't a Ygg::SimpleService::Email::Domain" unless object.is_a?(Ygg::SimpleService::Email::Domain)

          record = self.find_or_initialize_by(uuid: object.uuid)

          record.attributes = object.as_json(only: self.column_names - [:on_hold, :active]).symbolize_keys

          record.virtual = nil if object.virtual.blank?
          record.mx = 'mailbox.intercom.it' if object.mx.blank?

          record.save!

          fqdas = []

          object.aliases.limit(object.aliases.total_count).each do | fwd |
            fqdas << fwd.fqda if record.forwards.create_or_update_with_hel!(fwd)
          end

          db_fqdas = record.forwards.all.map(&:fqda)
          old = db_fqdas - fqdas
          record.forwards.where(fqda: old).map(&:destroy) if old.any?

          record
        end
      end
    end
  end
end
