
module MailAgent
  module Models
    module Policyd
      class AuthenticatedIdentity < Policyd::Base
        belongs_to :policy
        belongs_to :tracker

        def self.create_or_update_with_hel!(object)
          raise "#{object.to_s} isn't a Ygg::SimpleService::Email::RelayExtension" unless object.is_a?(Ygg::SimpleService::Email::RelayExtension)
          record = find_by_hel(object)
          record ||= find_by(identity: object.login)
          record.destroy! if record && !object.is_identity?
          return false unless object.is_identity?

          record ||= new
          record.identity  = object.login
          record.notes     = "hel:#{object.uuid}"
          record.disabled  = false
          record.static    = false
          record.policy_id = Policyd::Policy.find_policy_id(object.rate_limit)
          record.save!
          record
        end

        def self.find_by_hel(object)
          raise "#{object.to_s} isn't a Ygg::SimpleService::Email::RelayExtension" unless object.is_a?(Ygg::SimpleService::Email::RelayExtension)
          find_by(notes: "hel:#{object.uuid}")
        end

        after_destroy do
          self.tracker&.destroy!
        end

        before_create do          
          self.tracker = Tracker.create unless tracker
        end

        def disable!(reason)
          self.disabled = true
          self.disable_date = Time.now
          self.disable_reason = reason
          save!
        end

        def reset_tracker!
          transaction do
            tracker&.reset!
            self.disabled = false
            self.disable_date = nil
            self.disable_reason = nil
            save!
          end
        end
      end
    end
  end
end
