
module MailAgent
  module Models
    module Policyd
      class Policy < Policyd::Base
        has_many :authenticated_identities

        has_many :client_addresses

        POLICIES = %w(default default_for_addresses default_for_identities 128 512 1k 4k 12k trusted)
        OLD_POLICIES = {
          'usr_128_h' => '128',
          'usr_512_h' => '512',
          'usr_1k_h' =>  '1k',
          'usr_4k_h' =>  '4k',
          'usr_12k_h' => '12k',
        }

        def self.find_policy_id(policy)
          name = policy
          name = OLD_POLICIES[policy] if OLD_POLICIES.keys.include?(policy)
          name = nil unless POLICIES.include?(name)
          Policyd::Policy.find_by(name: name)&.id
        end

        def is_default?
          name.strip =~ /\Adefault/
        end
        
        def not_default?
          !is_default?
        end

        def self.find_by_hel(object)
          raise "#{object.to_s} isn't a Ygg::SimpleService::Email::RelayExtension" unless object.is_a?(Ygg::SimpleService::Email::RelayExtension)
          policy = self.where(name: object.rate_limit).first
        end

        def create_or_update_extension_with_hel!(object)
          raise "#{object.to_s} isn't a Ygg::SimpleService::Email::RelayExtension" unless object.is_a?(Ygg::SimpleService::Email::RelayExtension)
          if is_default?
            record = MailAgent::Models::Policyd::AuthenticatedIdentity.find_by_hel(object)
            record.destroy! if record
            record = MailAgent::Models::Policyd::ClientAddress.find_by_hel(object)
            record.destroy! if record
            false
          else
            rec1 = MailAgent::Models::Policyd::AuthenticatedIdentity.create_or_update_with_hel!(object, self)
            rec2 = MailAgent::Models::Policyd::ClientAddress.create_or_update_with_hel!(object, self)
            rec1 || rec2
          end
        end
      end
    end
  end
end