module Tomte::Workers
  class MailWorkerPubsub < Tomte::Worker

    bus :service, :enveloper => Tomte::Protocol::Enveloper::Generic do |ep|
      consumer = ep.consumer :queue => "ygg.email.events.#{Tomte.config[:local][:host]}", :ack => false
      consumer.connect! do |c|
        c.declare_exchange('ygg.email.events', :topic, :auto_delete => true) do
          c.bind('ygg.email.events')
          c.consume { log.info "ready to receive messages on ygg.email.events" }
        end

        consumer.prefetch 1

        log.debug '+ Sync database'
        @sync.sync
        @sync = nil
      end

      consumer.on_delivery do |metadata, message|
        log.debug "received message"
        log.debug message.ai
        ygg_obj = load_ygg_obj message
        case ygg_obj
        when Ygg::SimpleService::Email::Box
          case message['events']
          when /U/, /C/ then
            ygg_obj.create_or_update_db_box!
          when /D/ then
            ygg_obj.destroy_db_box!
          end
        when Ygg::SimpleService::Email::Domain
          case message['events']
          when /U/, /C/ then
            ygg_obj.create_or_update_db_domain!
          when /D/ then
            ygg_obj.destroy_db_domain!
          end
        when Ygg::SimpleService::Email::RelayExtension
          case message['events']
          when /U/, /C/ then
            ygg_obj.create_or_update_db_box!
            ygg_obj.create_or_update_db_extension!
          when /D/ then
            ygg_obj.destroy_db_box!
            ygg_obj.destroy_db_extension!
          end
        else
          log.error "* Unhandled object type #{ygg_obj.class.name}"
        end
      rescue 
        ex = $!
        if defined?(Raven)
          Raven.extra_context(message: message, metadata: metadata.attributes) do
            Raven.capture_exception(ex)
          end
        else
          raise ex
        end
      end
    end

    def init(options = {})
      log.debug '+ Fetch service data'
      @sync = ::MailAgent::BulkSync.new
    end

    protected

    def load_ygg_obj(data)
      if data[:model]
        klass = data[:model].constantize
        obj = klass.new(data[:object])
      end
      obj
    rescue 
      log.error '* ' +  $!.to_s
      log.error '* ' +  $!.backtrace.join("\n* ")
      nil
    end
  end
end
