module Tomte::Workers
  class MailWorkerRPC < Tomte::Worker
    
    class RPC < Tomte::Network::API::BasicServer
      def quotas(options = {})
        identity = options.delete('identity')
        address = options.delete('address')
        raise ArgumentError, "Must specify an identity or an address" unless address.present? || identity.present?
        object = if identity.present?
          MailAgent::Models::Policyd::AuthenticatedIdentity.find_by(identity: identity)
        elsif address.present?
          MailAgent::Models::Policyd::ClientAddress.find_by(address: address)
        end
        tracker = object&.tracker
        policy = object&.policy

        heuristics = (tracker&.heuristics || {})[:addresses]&.map do | k, v |
          v.merge(address: k)
        end

        {
          policy: policy&.name || 'unknown',
          disabled: !!object&.disabled,
          disable_reason: object&.disable_reason,
          disable_date: object&.disable_date,
          count: { quota: policy&.count || 0, count: tracker&.count || 0 },
          recipients: { quota: policy&.recipients || 0, count: tracker&.recipients || 0 },
          volume: { quota: policy&.volume || 0, count: tracker&.volume || 0 },          
          addresses: heuristics || [],
          last_update: tracker&.last_update
        }
      end

      def reset_quotas!(options = {})
        identity = options.delete('identity')
        address = options.delete('address')
        raise ArgumentError, "Must specify an identity or an address" unless address.present? || identity.present?
        
        object = if identity.present?
          MailAgent::Models::Policyd::AuthenticatedIdentity.find_by(identity: identity)
        elsif address.present?
          MailAgent::Models::Policyd::ClientAddress.find_by(address: address)
        end

        object&.reset_tracker!
      end

      def blacklisted_addresses(options = {})
        start = options[:start] || 0
        limit = options[:limit] || 0
        records = MailAgent::Models::Policyd::NetworkAddress.blacklisted
          .reorder(id: :asc)
          .limit(limit)
          .offset(start)
        
          records.as_json
      end

      def whitelisted_addresses(options = {})
        start = options[:start] || 0
        limit = options[:limit] || 0
        records = MailAgent::Models::Policyd::NetworkAddress.whitelisted
          .reorder(id: :asc)
          .limit(limit)
          .offset(start)
        
          records.as_json
      end
      
      def add_whitelist(address)
        addr = MailAgent::Models::Policyd::NetworkAddress.find_or_initialize_by(address: address)
        addr.flags = 'w'
        addr.notes = "Whitelisted"
        addr.inserted = Time.new
        addr.save!
      end
      
      def remove_network_address(address)
        addr = MailAgent::Models::Policyd::NetworkAddress.find_by(address: address)
        addr&.destroy!
        nil
      end
    end

    bus :service

    api :api, :bus => :service,
              :api => config.mail[:rpc_exchange],
              :class => RPC,
              :queue_options => { :auto_delete => true }
       
  end
end
